package edu.mit.harold.icollections;

import java.util.AbstractMap;
import java.util.AbstractSet;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;




/**
 * 
 * An immutable map from non-null keys to non-null values.
 * <p>
 * This map uses a given integer map to map hashcodes to lists of elements
 * with the same hashcode. Thus if all elements have the same hashcode, performance
 * is reduced to that of an association list.
 * 
 * @author harold
 *
 * @param <K>
 * @param <V>
 */
public final class HashIMap<K,V> extends AbstractMap<K,V> implements IMap<K,V> {
//// STATIC FACTORY METHODS ////
	/**
	 * @param <K>
	 * @param <V>
	 * @param intMap
	 * @return a map backed by an empty version of intMap,
	 * 	i.e. backed by intMap.minusAll(intMap.keySet())
	 */
	public static <K,V> HashIMap<K,V> empty(final IMap<Integer,ISequence<Entry<K,V>>> intMap) {
		return new HashIMap<K,V>(intMap.minusAll(intMap.keySet()), 0, 0); }
	

//// PRIVATE CONSTRUCTORS ////
	private final IMap<Integer,ISequence<Entry<K,V>>> intMap;
	private final int size, hashCode;
	// not externally instantiable (or subclassable):
	private HashIMap(final IMap<Integer,ISequence<Entry<K,V>>> intMap, final int size, final int hashCode) {
		this.intMap = intMap; this.size = size; this.hashCode = hashCode; }

	
//// REQUIRED METHODS FROM AbstractMap ////
	private Set<Entry<K,V>> entrySet = null;
	@Override
	public Set<Entry<K,V>> entrySet() {
		if(entrySet==null)
			entrySet = new AbstractSet<Entry<K,V>>() {
				// REQUIRED METHODS OF AbstractSet // 
				@Override
				public int size() {
					return size; }		
				@Override
				public Iterator<Entry<K,V>> iterator() {
					return new SequenceIterator<Entry<K,V>>(intMap.values().iterator()); }
				// OVERRIDDEN METHODS OF AbstractSet //
				@Override
				public int hashCode() { // same as Map
					return hashCode; }
				@Override
				public boolean contains(final Object e) {
					if(!(e instanceof Entry))
						return false;
					V value = get(((Entry<?,?>)e).getKey());
					return value!=null && value.equals(((Entry<?,?>)e).getValue());
				}
			};
		return entrySet;
	}

	
//// OVERRIDDEN METHODS FROM AbstractMap ////
	@Override
	public int hashCode() {
		return hashCode; }
	@Override
	public int size() {
		return size; }

	@Override
	public boolean containsKey(final Object key) {
		return keyIndexIn(getEntries(key.hashCode()), key) != -1; }
	
	@Override
	public V get(final Object key) {
		ISequence<Entry<K,V>> entries = getEntries(key.hashCode());
		for(Entry<K,V> entry : entries)
			if(entry.getKey().equals(key))
				return entry.getValue();
		return null;
	}

	
//// IMPLEMENTED METHODS OF IMap////
	public HashIMap<K,V> plusAll(final Map<? extends K, ? extends V> map) {
		HashIMap<K,V> result = this;
		for(Entry<? extends K,? extends V> entry : map.entrySet())
			result = result.plus(entry.getKey(), entry.getValue());
		return result;
	}

	public HashIMap<K,V> minusAll(final Collection<?> keys) {
		HashIMap<K,V> result = this;
		for(Object key : keys)
			result = result.minus(key);
		return result;
	}
	
	public HashIMap<K,V> plus(final K key, final V value) {
		ISequence<Entry<K,V>> entries = getEntries(key.hashCode());
		int size0 = entries.size(), hashCode0 = hashCode(entries),
			i = keyIndexIn(entries, key);
		if(i!=-1) entries = entries.minus(i);
		entries = entries.plus(new SimpleImmutableEntry<K,V>(key, value));
		return new HashIMap<K,V>(intMap.plus(key.hashCode(), entries),
				size-size0+entries.size(), hashCode-hashCode0+hashCode(entries));
	}

	public HashIMap<K,V> minus(final Object key) {
		ISequence<Entry<K,V>> entries = getEntries(key.hashCode());
		int i = keyIndexIn(entries, key);
		if(i==-1) // key not in this
			return this;
		int hashCode0 = hashCode(entries);
		entries = entries.minus(i);
		if(entries.size()==0) // get rid of the entire hash entry
			return new HashIMap<K,V>(intMap.minus(key.hashCode()),
					size-1, hashCode-hashCode0);
		// otherwise replace hash entry with new smaller one:
		return new HashIMap<K,V>(intMap.plus(key.hashCode(), entries),
				size-1, hashCode-hashCode0+hashCode(entries));
	}
	
	
//// PRIVATE UTILITIES ////
	private ISequence<Entry<K,V>> getEntries(final int hash) {
		ISequence<Entry<K,V>> entries = intMap.get(hash);
		if(entries==null) return ConsIStack.empty();
		return entries;
	}

	
//// PRIVATE STATIC UTILITIES ////
	private static <K,V> int keyIndexIn(final ISequence<Entry<K,V>> entries, final Object key) {
		int i=0;
		for(Entry<K,V> entry : entries) {
			if(entry.getKey().equals(key))
				return i;
			i++;
		}
		return -1;
	}
	
	private static <K,V> int hashCode(final ISequence<Entry<K,V>> entries) {
		int hash = 0;
		for(Entry<K,V> entry : entries)
			hash += entry.hashCode();
		return hash;
	}
	
	static class SequenceIterator<E> implements Iterator<E> {
		private final Iterator<ISequence<E>> i;
		private ISequence<E> seq = ConsIStack.empty();
		SequenceIterator(Iterator<ISequence<E>> i) {
			this.i = i; }

		public boolean hasNext() {
			return seq.size()>0 || i.hasNext(); }

		public E next() {
			if(seq.size()==0)
				seq = i.next();
			final E result = seq.get(0);
			seq = seq.subList(1, seq.size());
			return result;
		}

		public void remove() {
			throw new UnsupportedOperationException(); }
	}
}
